from jwt import decode
from django.contrib.auth import login, logout
from django.contrib.auth import get_user_model
from django.shortcuts import redirect

from url_auth.settings import url_auth_settings
from url_auth.validators import JWTPayloadvalidator


def login_by_url(request):
    '''
        It will decode the jwt and redirect to
        corresponding redirect_url passed in payload
    '''
    User = get_user_model()
    encoded_jwt = request.GET.get('enc', None)
    next_url = request.GET.get('next', None)
    if callable(request.user.is_authenticated):
        is_user_authenticated = request.user.is_authenticated()
    else:
        is_user_authenticated = request.user.is_authenticated
    if not all([next_url, encoded_jwt]):
        return redirect(next_url)
    try:
        payload = decode(
            encoded_jwt,
            url_auth_settings['SECRET_KEY'],
            algorithms=url_auth_settings['ENCRYPTION_ALGORITHIM']
        )
    except Exception:
        return redirect(next_url)
    try:
        JWTPayloadvalidator(payload)()
    except Exception:
        return redirect(next_url)
    user = User.objects.get(id=payload['userid'], is_active=True)
    login(request, user)
    return redirect(payload['redirect_url'])



from datetime import datetime, timedelta
try:
    from urllib.parse import urlencode
except Exception:
    from urllib import urlencode
import jwt

from django.contrib.auth import get_user_model
try:
    from django.urls import reverse
except Exception:
    from django.core.urlresolvers import reverse

from url_auth.settings import url_auth_settings
from url_auth.views import login_by_url


USER_MODEL = get_user_model()


def is_typeof(obj, expected_type):
    return type(obj) == expected_type


def generate_login_url(user_instance, *args, **kwargs):
    '''
        Return Unique URL for a user which can be use to log in the user.
    '''
    redirect_url = kwargs.get('redirect_url', '/')
    expiry_time_delta = kwargs.get(
        'expiry_time_delta', url_auth_settings['EXPIRY_TIME_DELTA'])

    if not is_typeof(user_instance, USER_MODEL):
        raise TypeError('User should be of type {}'.format(str(USER_MODEL)))
    if not is_typeof(redirect_url, str):
        raise TypeError('Resirect url should be of type{}'.format('str'))
    if not is_typeof(expiry_time_delta, timedelta):
        raise TypeError('expiry_time_delta should be of type{}'.format(
            str(timedelta)))

    payload = {
        'url_expiry_datetime': str(datetime.now() + expiry_time_delta),
        'userid': user_instance.id,
        'redirect_url': redirect_url,
        'time_stamp': str(datetime.now()),
    }

    jwt_token = jwt.encode(
        payload, url_auth_settings['SECRET_KEY'],
        algorithm=url_auth_settings['ENCRYPTION_ALGORITHIM'])

    return '{}?{}'.format(
        reverse(login_by_url),
        urlencode({
            'enc': jwt_token.decode('utf-8'),
            'next': redirect_url
        })
    )



from datetime import datetime

from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError


USER_MODEL = get_user_model()


class JWTPayloadvalidator(object):

    def __init__(self, payload):
        self.url_expiry_datetime = payload['url_expiry_datetime']
        self.userid = payload['userid']
        self.redirect_url = payload['redirect_url']

    def _validate_url_expiry_datetime(self):
        url_expiry_datetime = datetime.strptime(
            self.url_expiry_datetime, '%Y-%m-%d %H:%M:%S.%f')
        if url_expiry_datetime < datetime.now():
            raise ValidationError('Token Expired')

    def _validagte_userid(self):
        try:
            USER_MODEL.objects.get(id=self.userid, is_active=True)
        except USER_MODEL.DoesNotExist:
            raise ValidationError('Can not find user')

    def validate(self):
        self._validate_url_expiry_datetime()
        self._validagte_userid()

    def __call__(self):
        self.validate()



from datetime import timedelta
from django.conf import settings


USER_SETTINGS = getattr(settings, 'URL_AUTH_CONFIG', None)

DEFAULTS = {
    'SECRET_KEY': settings.SECRET_KEY,
    'EXPIRY_TIME_DELTA': timedelta(days=7),
    'ENCRYPTION_ALGORITHIM': 'HS256'
}

url_auth_settings = dict(
    DEFAULTS, **USER_SETTINGS) if USER_SETTINGS else DEFAULTS



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages


long_description = '''On an average, a person uses 60-90 applications, it’s not fun to enter passwords anymore.
Plus millions of passwords get exposed and accounts get compromised each year.
According to a survey, users admit they reuse the same password because it’s hard to remember them.
Of course, these problems are being addressed in relatively complex ways using 2FA, password managers
and such but a passwordless sign in is much simpler and quicker.'''

setup(
    name='urlauth',
    version='0.0.1',
    author='Arpit Srivastava',
    author_email='arpt.svt@gmail.com',
    packages=find_packages(),
    url='https://rpsingh_IN@bitbucket.org/rpsingh_IN/django-url-auth-package.git',
    license='MIT',
    description='Paswordless authentication',
    long_description=long_description,
    install_requires=['django', 'PyJWT'],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)
